/*
 * Mathematische Verfahren
 * Medien- und Kommunikationsinformatik
 * Hochschule Reutlingen
 * 
 * Copyright (c) 2010 Oliver Schumann
 *
 * created: 		13.05.2010
 * last modified: 	17.06.2010
 */

import java.util.ArrayList;
import java.util.Collections;
import java.util.SortedMap;
import java.util.TreeMap;

public class Huffman {
    
	private ArrayList<Node> nodeList = new ArrayList<Node>();

	private int[] chars = new int[256];

	private String plainText = "";
	private String codedText = "";

	private SortedMap<String, String> codeTable = new TreeMap<String, String>();
	private SortedMap<String, String> deCodeTable = new TreeMap<String, String>();
	private SortedMap<String, Double> probTable = new TreeMap<String, Double>();

	public Huffman() {
	}

	public Huffman(String plainText) {
		this.plainText = plainText;
	}

	public void countChars() {
		for (int i = 0; i < plainText.length(); i++) {
			int x = plainText.charAt(i);
			chars[x]++;
		}
	}

	public void createNodes() {
		for (int i = 0; i < 256; i++) {
			if (chars[i] > 0) {
				nodeList.add(new Node(Character.toString((char) i), chars[i]));
			}
		}
	}

	public void collectNodes() {
		Node node0 = nodeList.get(0);
		Node node1 = nodeList.get(1);

		nodeList.remove(node0);
		nodeList.remove(node1);

		nodeList.add(new Node(node0, node1));
	}

	public void createBitPattern() {
		nodeList.get(0).getChilds("", codeTable, deCodeTable);
	}

	public void codeText() {
		for (int i = 0; i < plainText.length(); i++) {
			codedText += codeTable.get(Character.toString(plainText.charAt(i)));
		}
	}

	public double getMiddleWordLength() {
		double mwl = 0.0;

		for (String key : codeTable.keySet()) {
			mwl += codeTable.get(key).length();
		}
		// return mwl / codeTable.size();
		return 1.0 * codedText.length() / plainText.length();
	}

	public void createProbTable() {
		int count = 0;
		double[] probTable = new double[chars.length];

		for (int i = 0; i < 256; i++) {
			if (chars[i] > 0) {
				probTable[i] = chars[i];
				count++;
			}
		}
		for (int i = 0; i < 256; i++) {
			if (probTable[i] > 0) {
				this.probTable.put(Character.toString((char) i), probTable[i]
						/ plainText.length());
			}
		}
	}

	public String deCodeText() {
		String returnString = new String();
		String codedString = codedText.toString();

		while (codedString.length() > 0) {
			String s = "";
			int i = 1;

			while (!deCodeTable.containsKey(s)) {
				s = codedString.substring(0, i++);
				if (deCodeTable.containsKey(s)) {
					codedString = codedString.substring(i - 1);
					returnString += deCodeTable.get(s);
				}
			}
		}
		return returnString;
	}

	public void compress() {

		while (nodeList.size() > 1) {
			Collections.sort(nodeList);
			collectNodes();
		}
	}

	@Override
	public String toString() {
		return "Huffmann [getClass()=" + getClass() + ", hashCode()="
				+ hashCode() + ", toString()=" + super.toString() + "]";
	}

	public String getPlainText() {
		return plainText;
	}

	public void setPlainText(String plainText) {
		this.plainText = plainText;
	}

	public String getCodedText() {
		return codedText;
	}

	public void setCodedText(String codedText) {
		this.codedText = codedText;
	}

	public int[] getChars() {
		return chars;
	}

	public int getCharAt(int i) {
		return chars[i];
	}

	public SortedMap<String, String> getCodeTable() {
		return codeTable;
	}

	public SortedMap<String, String> getDeCodeTable() {
		return deCodeTable;
	}

	public SortedMap<String, Double> getProbTable() {
		return probTable;
	}

	public void doAll() {
		countChars();
		createProbTable();
		createNodes();
		compress();
		createBitPattern();
		codeText();
	}

	public Double calcEntropie() {
		Double result = new Double(0.0);
		for (String key : probTable.keySet()) {
			result += probTable.get(key) * log2(probTable.get(key));
		}
		return -result;

	}

	private double log2(double x) {
		return Math.log(x) / Math.log(2);
	}

}