/*
 * Mathematische Verfahren
 * Medien- und Kommunikationsinformatik
 * Hochschule Reutlingen
 * 
 * Copyright (c) 2010 Oliver Schumann
 *
 * created: 		13.05.2010
 * last modified: 	09.07.2010
 */


import java.applet.Applet;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;

public class HuffmanApplet extends Applet implements ActionListener {

	private static final long serialVersionUID = 1L;
	TextField txtPlainText, txtCodedText, txtDeCodedText, txtEntropy, txtMidWordLen;
	TextArea charTable, probTable, codeTable, deCodeTable;
	Checkbox allresults;
	Button codeButton, deCodeButton, resetButton;
	
	DecimalFormat f = new DecimalFormat("#0.000"); 
	
	Huffman huffman;
	// layout
	public void init() {
		setLayout(new GridBagLayout());
		GridBagConstraints c;

		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 2;
		add(codeButton = new Button(("encode")), c);
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 1;
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 2;
		add(deCodeButton = new Button(("decode")), c);
		
		c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 2;
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 2;
		add(resetButton = new Button(("reset")), c);

		c = new GridBagConstraints();
		c.gridx = 2;
		c.gridy = 0;
		c.anchor = GridBagConstraints.EAST;
		add(new Label("plaintext"), c);

		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		add(txtPlainText = new TextField("MISSISSIPPI", 55), c);

		c = new GridBagConstraints();
		c.gridx = 2;
		c.gridy = 1;
		c.anchor = GridBagConstraints.EAST;
		add(new Label("ciphertext"), c);

		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 1;
		c.anchor = GridBagConstraints.WEST;
		add(txtCodedText = new TextField("", 55), c);
		txtCodedText.setEditable(false);
 
		c = new GridBagConstraints();
		c.gridx = 2;
		c.gridy = 5;
		c.anchor = GridBagConstraints.EAST;
		add(new Label("      average word length"), c);

		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 5;
		c.anchor = GridBagConstraints.WEST;
		add(txtMidWordLen = new TextField("", 10), c);
		txtMidWordLen.setEditable(false);
		c = new GridBagConstraints();
		c.gridx = 2;
		c.gridy = 6;
		c.anchor = GridBagConstraints.EAST;
		add(new Label("entropy"), c);

		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 6;
		c.anchor = GridBagConstraints.WEST;
		add(txtEntropy = new TextField("", 10), c);
		txtEntropy.setEditable(false);
		
		c = new GridBagConstraints();
		c.gridx = 2;
		c.gridy = 3;
		c.anchor = GridBagConstraints.WEST;
		add(new Label("         frequency"), c);
		c = new GridBagConstraints();
		c.gridx = 2;
		c.gridy = 4;
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 5;
		add(charTable = new TextArea(15, 15 ), c);
		charTable.setEditable(false);

		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 3;
		c.anchor = GridBagConstraints.WEST;
		add(new Label("         probability"), c);
		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 4;
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 5;
		add(probTable = new TextArea(15, 15 ), c);
		probTable.setEditable(false);
		
		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 3;
		c.anchor = GridBagConstraints.CENTER;
		add(new Label("encode-table"), c);
		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 4;
		c.anchor = GridBagConstraints.CENTER;
		c.gridwidth = 5;
		add(codeTable = new TextArea(15, 15), c);
		codeTable.setEditable(false);
		
		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 3;
		c.anchor = GridBagConstraints.EAST;
		add(new Label("decode-table      "), c);
		
		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 4;
		c.anchor = GridBagConstraints.EAST;
		c.gridwidth = 5;
		add(deCodeTable = new TextArea(15, 15), c);
		deCodeTable.setEditable(false);
		
		c = new GridBagConstraints();
		c.gridx = 2;
		c.gridy = 2;
		c.anchor = GridBagConstraints.EAST;
		add(new Label("plaintext"), c);

		c = new GridBagConstraints();
		c.gridx = 3;
		c.gridy = 2;
		c.anchor = GridBagConstraints.WEST;
		add(txtDeCodedText = new TextField("", 55), c);
		txtDeCodedText.setEditable(false);
		
		// register listeners
		txtPlainText.addActionListener(this);
		txtCodedText.addActionListener(this);
		codeButton.addActionListener(this);
		deCodeButton.addActionListener(this);
		resetButton.addActionListener(this);
		
		deCodeButton.setEnabled(false);


	}

	// listen, dispatch
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == codeButton) {
			doCompress();
		}
		
		if (e.getSource() == deCodeButton) {
			doDeCompress();
		}
		
		if (e.getSource() == resetButton) {
			reset();
		}
	}
  
	public void doCompress() {
		
		if (txtPlainText.getText().length() != 0) {
		huffman = new Huffman();
		huffman.setPlainText(txtPlainText.getText());
		huffman.doAll();
		txtCodedText.setText(huffman.getCodedText());
		charTable.setText("");
		codeTable.setText("");
		deCodeTable.setText("");
		
		for (int i = 0; i < huffman.getChars().length; i++) {
			if(huffman.getCharAt(i) > 0){
				charTable.append((char) i + "\t" + huffman.getCharAt(i) + "\n");
			}
			
		}
		for (String key : huffman.getCodeTable().keySet()) {
			codeTable.append(key + "\t" + huffman.getCodeTable().get(key) + "\n");
		}
		for (String key : huffman.getDeCodeTable().keySet()) {
			deCodeTable.append(key + "\t" + huffman.getDeCodeTable().get(key) + "\n");
		}
		txtEntropy.setText(f.format(huffman.calcEntropie()));
		txtMidWordLen.setText(f.format(huffman.getMiddleWordLength()));
		
		probTable.setText("");
		for (String key : huffman.getProbTable().keySet()) {
			probTable.append(key + "\t" + f.format(huffman.getProbTable().get(key)) + "\n");
		}
		deCodeButton.setEnabled(true);
		codeButton.setEnabled(false);
		}else
		{
			txtCodedText.setText("error, no text to encode!");
		}
	}
	
	public void doDeCompress() {
		txtDeCodedText.setText(huffman.deCodeText());
		deCodeButton.setEnabled(false);
	}
	
	public void reset() {
		charTable.setText("");
		txtPlainText.setText("");
		txtDeCodedText.setText("");
		txtCodedText.setText("");
		codeTable.setText("");
		probTable.setText("");
		deCodeTable.setText("");
		deCodeButton.setEnabled(false);
		codeButton.setEnabled(true);
	}
}
